local VERSION = tonumber(("$Revision: 43948 $"):match("%d+"))

local Parrot = Parrot
local Parrot_Triggers = Parrot:NewModule("Triggers", "AceEvent-2.0", "Parser-3.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-16 17:54:49 -0400 (Mon, 16 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_Triggers")

local new, del, newSet = Parrot.new, Parrot.del, Parrot.newSet

local _,playerClass = UnitClass("player")

Parrot_Triggers.db = Parrot:AcquireDBNamespace("Triggers")
Parrot:RegisterDefaults("Triggers", 'profile', {})

local registry = {
	{
		name = L["Low Health!"],
		class = "WARLOCK;WARRIOR;MAGE;ROGUE;DRUID;PALADIN;SHAMAN;PRIEST;HUNTER",
		triggers = {
			["Self Health Percent"] = 0.4,
		},
		secondaryTriggers = {
			["Trigger Cooldown"] = 3,
		},
		scrollArea = "Notification",
		sticky = true,
		color = "ff7f7f",
	},
	{
		name = L["Low Mana!"],
		class = "WARLOCK;MAGE;DRUID;PALADIN;SHAMAN;PRIEST;HUNTER",
		triggers = {
			["Self Mana Percent"] = 0.35,
		},
		secondaryTriggers = {
			["Trigger Cooldown"] = 3,
		},
		scrollArea = "Notification",
		sticky = true,
		color = "7f7fff",
	},
}

local effectiveRegistry = {}
local function rebuildEffectiveRegistry()
	for _,v in ipairs(registry) do
		local classes = newSet((';'):split(v.class))
		if classes[playerClass] then
			effectiveRegistry[#effectiveRegistry+1] = v
		end
		classes = del(classes)
	end
end

local cooldowns = {}
local currentTrigger
local Parrot_TriggerConditions
local Parrot_Display
function Parrot_Triggers:OnEnable(first)
	Parrot_TriggerConditions = Parrot:GetModule("TriggerConditions")
	if first then
		Parrot_TriggerConditions:RegisterSecondaryTriggerCondition {
			name = "Trigger Cooldown",
			localName = L["Trigger Cooldown"],
			defaultParam = 3,
			param = {
				type = "range",
				min = 0,
				max = 60,
				step = 0.1,
				bigStep = 1,
			},
			check = function(param)
				if not cooldowns[currentTrigger] then
					return true
				end
				local now = GetTime()
				return now - cooldowns[currentTrigger] > param
			end,
		}
		
	end
	
	Parrot_Display = Parrot:GetModule("Display")
	rebuildEffectiveRegistry()
end

function Parrot_Triggers:OnTriggerCondition(name, arg)
	for _,v in ipairs(effectiveRegistry) do
		local triggers = v.triggers
		if triggers then
			local param = triggers[name]
			if param then
				local good = false
				if param == true then
				 	good = true
				elseif type(arg) == "string" then
					good = param == arg
				elseif type(arg) == "number" then
					good = arg <= param
				end
				if good then
					local secondaryTriggers = v.secondaryTriggers
					if secondaryTriggers then
						currentTrigger = v.name
						for k, v in pairs(secondaryTriggers) do
							if not Parrot_TriggerConditions:DoesSecondaryTriggerConditionPass(k, v) then
								good = false
								break
							end
						end
					end
					if good then
						cooldowns[v.name] = GetTime()
						Parrot_Display:ShowMessage(v.name, v.scrollArea, v.sticky, v.color, v.font, v.fontSize, v.outline)
					end
				end
			end
		end
	end
end
